/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["ResolveRedir"];

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://sublink-migreme-modules/util.js");


function ResolveRedir() {
  this.onRedirFound = null;
  this.onError = null;
  this.onLoad = null;
  this._counter = 0;
}


ResolveRedir.prototype = {

  findRedir: function(uri) {
    var me = this;
    var xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    xhr.mozBackgroundRequest = true;
    xhr.open("HEAD", uri.spec, true);
    xhr.setRequestHeader("Accept", null);
    xhr.setRequestHeader("Accept-Language", null);
    xhr.setRequestHeader("Accept-Encoding", null);
    xhr.setRequestHeader("Cookie", null);
    xhr.setRequestHeader("User-Agent", "SubLink/4.0 (+http://br.mozdev.org/migre.me/)");
    // LOAD_ONLY_IF_MODIFIED makes xhr results from cache be empty!
    //xhr.channel.loadFlags |= Ci.nsICachingChannel.LOAD_ONLY_IF_MODIFIED;


    xhr.channel.notificationCallbacks = {
      QueryInterface: XPCOMUtils.generateQI([Ci.nsIChannelEventSink]),

      getInterface: function(aIID) {
        try {
          return this.QueryInterface(aIID);
        } catch (ex) {
          throw Components.results.NS_NOINTERFACE;
        }
      },

      asyncOnChannelRedirect: function(aOldChannel, aNewChannel, aFlags, callback) {
        var diff = (new Date().getTime()) - me._start;
        me._counter += 1;
        callback.onRedirectVerifyCallback(Components.results.NS_FAILED);
        xhr.abort(); // onRedirectVerifyCallback doesn't seem to have any effect

        console.log('===== asyncOnChannelRedirect');
        console.log('OLD:', aOldChannel.originalURI.spec, aOldChannel.URI.spec);
        console.log('NEW:', aNewChannel.originalURI.spec, aNewChannel.URI.spec);
        //console.log(aOldChannel.QueryInterface(Ci.nsIHttpChannel).responseText); undefined
        //console.log(aNewChannel.QueryInterface(Ci.nsIHttpChannel).responseText); no interface

        try {
          var redirCode = aOldChannel.QueryInterface(Ci.nsIHttpChannel).responseStatus;
        } catch (ex) {
          // old url=new?
          //[Exception... "Component returned failure code: 0x80040111 (NS_ERROR_NOT_AVAILABLE) [nsIHttpChannel.responseStatus]"  nsresult: "0x80040111 (NS_ERROR_NOT_AVAILABLE)"  location: "JS frame :: resource://sublink-migreme-modules/resolve-redir.js :: <TOP_LEVEL> :: line 66"  data: no]
          //callback.onRedirectVerifyCallback(Components.results.NS_FAILED);
          Components.utils.reportError(ex);
          return;
        }

        me.onRedirFound({
          delta:              diff,
          longUri:            aNewChannel.URI,
          responseStatusCode: redirCode
        }, me._counter);
      }
    };

    xhr.onerror = function(evt) {
      console.log('XHR ERROR', evt);
      me.onError({
        id: "noconnection",
        xhr: xhr,
        msg: null
      });
    };
    xhr.onload = function(evt) { // RedirServer.isRedirUri = false???
      console.log('XHR ONLOAD', evt);
      var diff = new Date().getTime() - me._start;
      var channel = evt.target.channel;
      me.onLoad({
          delta:              diff,
          longUri:            channel.URI,
          responseStatusCode: channel.responseStatus
        });
    };
    xhr.onabort = function(evt) { // called by xhr.abort();
      //console.log('XHR ABORT', evt, this, evt.target, evt.currentTarget );//[object ProgressEvent]
    };
      // BUG sem redir (eg http://migre.me/p/limpa_brasil) precisa atualizar menuitem
    xhr.onloadend = function(evt) {
    };

    me._start = new Date().getTime();
    xhr.send(null);
  }

  /*
  _load: function(xhr2, uriRequest, timespan) {
    if (xhr2.status === 200) {
      var uri = getRefresh(xhr2);
      if (uri) {
        this.onRedirFound({
          delta:              timespan,
          longUri:            uri,
          responseStatusCode: xhr2.status
        });
        return;
      }

      // error, redirectionLimit != 0?
      if (xhr2.channel.URI.equals(uriRequest) === false) {
        this.onRedirFound({
          delta:               timespan,
          longUri:             xhr2.channel.URI,
          responseStatusCode: -200
        });
        return;
      }
    }


    console.log(xhr2.status + ' - ' + xhr2.statusText);
    if (xhr2.status > 400) {
      this.onError({
        id: "couldntrestore", // server-error?
        xhr: xhr2,
        msg: xhr2.status + ' - ' + xhr2.statusText
      });
    } else {
      console.log("response:" + xhr2.channel.QueryInterface(Ci.nsIHttpChannel).requestMethod + " " + xhr2.channel.originalURI.spec + "\n"
        + xhr2.responseText.substr(0, 500));
      this.onError({
        id: "couldntrestore-short",
        xhr: xhr2,
        msg: null
      });
    }
  },


  _error: function(xhr2, timespan) {

    switch (xhr2.status) {
      case 301:
      case 302:
        break;
      default: // 0 = no response
        this.onError({
          id: "noconnection",
          xhr: xhr2,
          msg: null
        });
        return;
    }

    // status ok

    var uri = resolveUrl(xhr2.getResponseHeader("Location"),
                         xhr2.channel.originalURI);
    if (uri === null) {
      console.log("Location: " + uri.spec);
      this.onError({
        id: "invalid",
        xhr: xhr2, // BUG xhr will be empty
        msg: null
      });
      return;
    }

    this.onRedirFound({
      delta:              timespan,
      longUri:            uri,
      responseStatusCode: xhr2.status
    });
  }
  */


};

/*
function getRefresh(xhr) {
  var header = xhr.getResponseHeader("Refresh");
  if (header === null) {
    return null;
  }
  var fields = header.split(";", 2);       // ["0 ", " url=http "]
  if (fields.length !== 2) {
    return null;
  }

  var nameValue = fields[1].split("=", 2); // [" url", "http "]
  if (nameValue.length !== 2) {
    return null;
  }

  if (nameValue[0].trim().toLowerCase() !== "url") {
    return null;
  }

  return util.stringToUri(nameValue[1].trim());
}
*/

/*
function resolveUrl(url, uri) {
  if (url === null) {
    return null;
  }
  if (url.indexOf("/") === 0) {
    url = uri.resolve(url);
  }
  return util.stringToUri(url);
}
*/

/*
  getLocationHeader: function(xhr2) {

    var url = xhr2.getResponseHeader("Location");
    if (url === null) {
      return null;
    }

    var uri;
    if (url.indexOf("/") === 0) { // url="/path"
      uri = xhr2.channel.originalURI.clone();
      uri.path = url;
    } else {
      uri = util.stringToUri(url);
    }

    return uri;
  },
*/

