/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["RedirServer", "convertUrl"];

Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import("resource://sublink-migreme-modules/util.js");


function convertUrl(uri, onSuccess, error, complete) {
  if (RedirServer.isRedirUri(uri)) {
    console.trace(uri.spec);
  }

  Components.utils.import("resource://sublink-migreme-modules/api-migreme.js");
  var req = new AsyncRequest();

  req.onError = error;

  req.onSuccess = function(info) {
    onSuccess(info);
    Components.utils.import("resource://sublink-migreme-modules/redir-storage.js");
    redirStorage.saveShortUrl(uri, info.shortUri);
  };

  var start = new Date().getTime();
  req.onComplete = function() {
    if (complete) {
      complete();
    }
    console.log("Connection migre.me:", (new Date().getTime() - start) + "ms");
  };

  req.convert(uri);
}



var RedirServer = {

  isRedirUri: function(uri) {
    switch (uri.scheme) {
      case "http":
      case "https":
        break;
      default:
        return false;
    }

    switch (uri.host) {
      case "tumblr.com":
      case "feeds.gawker.com":
      case "feeds.feedburner.com":
      case "feeds.wired.com":
      case "ff.im":
        return true;
      case "digg.com":
        return uri.path.indexOf("/", 1) === -1;
    }



    var baseDomain;
    try {
      baseDomain = Services.eTLD.getBaseDomain(uri);
    } catch (ex) {
      return false; //e.g.: localhost
    }

    // no path
    if (uri.path.length === 1) { // "/".length=1
      var publicSuffix = Services.eTLD.getPublicSuffix(uri);
      if (publicSuffix === "tk") { // http://ura71.tk
        return true;
      }

      switch (baseDomain) {
        case "th8.us":
          return true;
      }
      return false;
    }

    if (RedirServer.isBitLy(baseDomain)) {
      return true;
    }

    switch (baseDomain) {
    // path=/xxxx/
    //if (uri.path.indexOf("/", 1) > 0) {
    //  switch (baseDomain) {
      case "flickr.com":
        return uri.path.indexOf("/p/") === 0;

      case "google.com":
        return uri.host === "feedproxy.google.com";
      //}
      //return false;
    //}
    //switch (baseDomain) { // todo baseDomain OR www + baseDomain

      case "th8.us":
        return uri.path.indexOf("/forw") === 0; // http://th8.us/forward.php?xyz

      case "blip.fm":
        return uri.path.indexOf("/~") === 0 || uri.path.indexOf("/%7E") === 0;

      case "g.co":
      case "goo.gl":
      case "youtu.be":
      case "shar.es":
      case "me.lt":
      case "arst.ch":
      case "dxurl.com":
      case "deck.ly":
      case "abr.io":
      case "gd.is":
      case "dlvr.it":
      case "fb.me":
      case "ru.ly":
      case "t.co":
      case "heello.me":
      case "is.gd": // BUG com http://is.gd/0CJEZ0?t=28159
      case "v.gd":
      case "im.ly":
      case "3.ly":  // "3.ly/aaa" ==> "3.ly/aaa/" ==> url
      case "flic.kr":
      case "uol.com":
      case "ig.com":
      case "bugzil.la":
      case "crbug.com":
      case "migre.me":
      case "migreme.com":
      case "senta.la":
      case "kingo.to":
      case "virou.gr":
      case "pqp.vc":
      case "curt.im":
      case "pud.im":
      case "va.mu":
      case "r2.ly":
      case "digs.by":
      case "nyt.com":
      case "zapt.in":
      case "ustre.am":
      case "htxt.it":
      case "om.ly":
      case "p.ly":
      case "fwd4.me":
      case "2tu.us":
      case "adjix.com":
      case "alturl.com":
      case "b23.ru":
      case "bacn.me":
      case "bloat.me":
      case "budurl.com":
      case "chilp.it":
      case "cli.gs":
      case "clipurl.us":
      case "cort.as":
      case "dwarfurl.com":
      case "fff.to":
      case "fit.am":
      case "href.in":
      case "icio.us":
      case "idek.net":
      case "kiq.me":
      case "kl.am":
      case "korta.nu":
      case "lin.cr":
      case "loopt.us":
      case "lost.in":
      case "memurl.com":
      case "merky.de":
      case "mltp.ly":
      case "moby.to":
      case "moourl.com":
      case "nanourl.se":
      case "notlong.com":
      case "okm.me":
      case "ow.ly":
      case "peaurl.com":
      case "ping.fm":
      case "piurl.com":
      case "plurl.me":
      case "pnt.me":
      case "poprl.com":
      case "post.ly":
      case "qik.ly":
      case "qlnk.net":
      case "rde.me":
      case "redir.ec":
      case "retwt.me":
      case "rubyurl.com":
      case "short.ie":
      case "short.to":
      case "shorten.ws":
      case "siga.st":
      case "smallr.com":
      case "sn.im":
      case "sn.vc":
      case "snipr.com":
      case "snipurl.com":
      case "snurl.com":
      case "su.pr":
      case "tiny.cc":
      case "tinysong.com":
      case "tinyurl.com":
      case "tl.gd":
      case "togoto.us":
      case "tra.kz":
      case "trg.li":
      case "trim.li":
      case "trunc.it":
      case "twa.lk":
      case "twurl.cc":
      case "twurl.nl":
      case "u.nu":
      case "ur1.ca":
      case "url.az":
      case "url.ie":
      case "url4.eu":
      case "urlx.ie":
      case "vai.la":
      case "w34.us":
      case "wp.me":
      case "xrl.us":
      case "xurls.info":
      case "yep.it":
      case "yumurl.com":
      case "zi.ma":
      case "zip.li":
      case "zurl.ws":

      case "xn--hgi.ws": // ➡
      case "xn--ogi.ws": // ➨
      case "xn--vgi.ws": // ➯
      case "xn--3fi.ws": // ➔
      case "xn--egi.ws": // ➞
      case "xn--9gi.ws": // ➽
      case "xn--5gi.ws": // ➹
      case "xn--1ci.ws": // ✩
      case "xn--odi.ws": // ✿
      case "xn--rei.ws": // ❥
      case "xn--cwg.ws": // ›
      case "xn--bih.ws": // ⌘
      case "xn--fwg.ws": // ‽
      case "xn--l3h.ws": // ☁
      case "ta.gd":

        return true;
    }

    return false;
  },

  isBitLy: function(tld) {
    switch (tld) {
      case "4ms.me":
      case "abcn.ws":
      case "adobe.ly":
      case "amzn.to":
      case "aol.it":
      case "bbc.in":
      case "bit.ly": // xxx.bit.ly/123/dddd
      case "qr.ae":
      case "cnet.co":
      case "cot.ag":
      case "es.pn":
      case "tnw.co":
      case "gaw.kr":
      case "gizmo.do":
      case "glo.bo":
      case "go.nasa.gov":
      case "intel.ly":
      case "j.mp":
      case "lifehac.kr":
      case "lnkd.in":
      case "mzl.la":
      case "nyti.ms":
      case "onion.com":
      case "read.bi":
      case "reut.rs":
      case "tcrn.ch":
      case "ti.me":
      case "yhoo.it":
      case "zd.net":
      case "zite.to":
      case "econ.st":
        return true;
    }
    return false;
  },


  getInfoUrl: function(uri) {
    var baseDomain = Services.eTLD.getBaseDomain(uri);

    if (RedirServer.isBitLy(baseDomain)) {
      return uri.spec + "+";
    }

    switch (baseDomain) {
      case "migreme.com":
      case "migre.me": // http://migre.me/ret/xxx
        var code = uri.path.substr(uri.path.lastIndexOf("/") + 1);
        return "http://migre.me/contar-cliques/?url="  + encodeURIComponent(code);
      case "goo.gl":
      case "va.mu":
      case "p.ly":
        return uri.spec + "+";
      case "zapt.in":
        return "http://zapt.in/short/" + encodeURIComponent(uri.path.substr(1));
      case "3.ly":
        if (uri.path.lastIndexOf("/") === (uri.path.length - 1)) {
          return null;
        } else {
          return uri.spec + "-";
        }
      case "is.gd":
      case "v.gd":
        return uri.spec + "-";
      case "th8.us":
        if (uri.host !== baseDomain) { // "xxxxx.th8.us" vs. "th8.us"
          var uri2 = uri.clone();
          uri2.host = "p." + uri.host;
          return uri2.spec;
        }
        break;
    }

    return null;
  }

};
